#!/bin/sh

# Include library scripts.
if [ -f "/opt/drweb.com/share/scripts/include/lib.sh" ]; then
    . "/opt/drweb.com/share/scripts/include/lib.sh"
elif [ -f "/usr/local/libexec/drweb.com/share/scripts/include/lib.sh" ]; then
    . "/usr/local/libexec/drweb.com/share/scripts/include/lib.sh"
else
    printf '\nScript library not found.\n\n'
    exit 1
fi

checkRoot

SelectShares() {
    # Check is there are some smb shares at all
    if [ $# -eq 0 ]; then
        echo
        echo "There are no SMB shares configured in your smb.conf."
        return 0
    fi
    numshares=`echo "$*" | wc -l`

    # Set defaults.
    key='vfs objects'
    keyre='vfs[[:space:]][[:space:]]*objects\{0,1\}'

    for n in `Seq 1 $numshares`; do
        share=`echo "$*" | head -n $n | $TAIL -n 1`
        val=`IniGetValue "$SMB_CONF_FILE" "$share" "$keyre"`
        if StringInList "$val" smb_spider; then
            eval "a_$n=1"
        else
            eval "a_$n="
        fi
    done

    # Poll loop.
    while true; do
        clear
        echo
        echo "Samba conf file: \"$SMB_CONF_FILE\""
        echo
        echo "Select SMB shares the smbspider will be connected to."
        echo
        for n in `Seq 1 $numshares`; do
            share=`echo "$*" | head -n $n | $TAIL -n 1`
            printf "%3d) " $n
            r=`eval "echo \\$a_$n"`
            if [ -n "$r" ]; then
                printf "[X]"
            else
                printf "[ ]"
            fi
            echo " $share"
        done
        echo
        echo "Enter directory number to toggle selection."
        echo "Enter A or All to select all directories."
        echo "Enter N or None to deselect all directories."
        echo "Enter 0, Q or Quit when done. "
        echo "All values are case insensitive."
        printf "Select: "
        read inp
        case "$inp" in
            0 | [qQ] | [qQ][uU][iI][tT])
                break
                ;;
            [aA][lL][lL] | [aA])
                for n in `Seq 1 $numshares`; do
                    eval "a_$n=1"
                done
                continue
                ;;
            [nN][oO][nN][eE] | [nN])
                for n in `Seq 1 $numshares`; do
                    eval "a_$n="
                done
                continue
                ;;
        esac

        r=`eval "echo \\$a_$inp"`
        if [ -z "$r" ]; then
            eval "a_$inp=1"
        else
            eval "a_$inp="
        fi
    done

    SMB_CONF_FILE_NEW=`MakeTmpFile`
    cp -p -f $SMB_CONF_FILE $SMB_CONF_FILE_NEW

    for n in `Seq 1 $numshares`; do
        share=`echo "$*" | head -n $n | $TAIL -n 1`
        share_on=`eval "echo \\$a_$n"`
        val=`IniGetValue "$SMB_CONF_FILE_NEW" "$share" "$keyre"`
        StringInList "$val" smb_spider
        using_smbs=$?

        if [ -n "$share_on" ]; then # Need to turn it on
            # Check if it's using smbspider already => continue loop.
            [ "$using_smbs" = "0" ] && continue

            # If there's "vfs object" key in config, append "smb_spider"
            if [ -n "$val" ]; then
                val="${val} smb_spider"
            else
                val=smb_spider
            fi
            IniSetValue "$SMB_CONF_FILE_NEW" "$share" "$key" "$keyre" "$val" "Added by Dr.Web's `basename $0`"
        else # If need to turn it off:
            # Check if it's turned on.
            if [ "$using_smbs" = "0" ]; then
                # Remove "smb_spider" from the line.
                val=`echo "$val" | $SED "s/smb_spider[[:space:]]*//"`
                IniSetValue "$SMB_CONF_FILE_NEW" "$share" "$key" "$keyre" "$val"
            fi
        fi
    done

    AskUpdateConfig $SMB_CONF_FILE $SMB_CONF_FILE_NEW
}

FindSambaConfig() {
    SMB_CONF_FILE=`$SMBD_PATH -b | grep CONFIGFILE 2>/dev/null | awk '{print $2}'`
    if [ -z "$SMB_CONF_FILE" ]; then
        echo
        echo "Error: No CONFIGFILE has been reported by \"$SMBD_PATH\"."
        return 1
    elif [ ! -f "$SMB_CONF_FILE" ]; then
        echo
        echo 'Error: Samba configuration file not found!'
        echo
        exit 1
    else
        return 0
    fi
}

FindSambaBins() {
    bins=""
    for p in /opt/csw /usr /usr/local /usr/local/samba /opt/samba ; do
        bins="$bins $p/*bin/smbd"
    done
    RET=
    for b in $bins; do
        [ -x "$b" ] && RET="$RET $b"
    done

    echo $RET
}

SmbSpiderSetup() {
    if showyesno "Do you want your smb.conf to be patched now?" ; then
        true
    else
        echo
        exit 0
    fi

    # Find smbd.
    #
    smbbins=`FindSambaBins`

    while true; do
        if [ "`echo $smbbins | wc -w`" -eq "0" ]; then
            echo
            echo "Samba daemon not found!"
        elif [ "`echo $smbbins | wc -w`" -eq "1" ]; then
            echo
            if showyesno "Is \"$smbbins\" the Samba daemon in use?"; then
                SMBD_PATH=$smbbins
                FindSambaConfig && break
            fi
        else
            echo
            echo "The following Samba binaries were found:"
            echo
            for b in $smbbins; do
                echo $b
            done
        fi

        echo
        echo "Please enter the path to your Samba daemon executable (smbd)"
        printf "(enter 0, Q or Quit to cancel): "
        read SMBD_PATH
        case "$SMBD_PATH" in
            0 | [qQ] | [qQ][uU][iI][tT])
                exit 0
                ;;
        esac
        if [ ! -x "$SMBD_PATH" ]; then
            echo
            echo $SMBD_PATH is not executable.
            SMBD_PATH=
            continue
        fi

        # Find smb.conf.
        FindSambaConfig && break
    done

    # Retrieve shares from smb.conf and let the user select some.
    SelectShares "`$AWK '/^[ \t]*\[.*\][ \t]*\$/{ gsub(/^[ \t]*\[|\][ \\t]*/, ""); print(\$0) }' < "${SMB_CONF_FILE}" | grep -v '^global$'`"
}

SmbSpiderSetup

# Update Samba symbolic links.
echo
echo "Updating links for \"$SMBD_PATH\"..."
$PREFIX/share/drweb-smbspider-modules/update-links.sh -s $SMBD_PATH #>/dev/null
if [ "$?" -ne "0" ]; then
    echo "update-links.sh failed."
    exit 1
fi

echo
echo "Configuration of drweb-smbspider is completed successfully."
echo
exit 0
